using CoreGraphics;
using Syncfusion.SfDataGrid;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UIKit;

namespace FilteringDemo
{
    public class Filtering : UIViewController
    {

        #region Fields

        SfDataGrid SfGrid;
        UISearchBar searchbar;
        OptionsView option;
        private FilterViewModel viewmodel;

        #endregion

        static bool UserInterfaceIdiomIsPhone
        {
            get { return UIDevice.CurrentDevice.UserInterfaceIdiom == UIUserInterfaceIdiom.Phone; }
        }

        public Filtering()
        {
            viewmodel = new FilterViewModel();

            this.SfGrid = new SfDataGrid();
            this.SfGrid.SelectionMode = SelectionMode.Single;
            this.SfGrid.AutoGeneratingColumn += GridAutoGenerateColumns;
            this.SfGrid.ItemsSource = viewmodel.BookInfo;
            this.SfGrid.HeaderRowHeight = 45;
            this.SfGrid.RowHeight = 45;
            searchbar = new UISearchBar();
            searchbar.OnEditingStarted += HandleOnEditingStarted;
            searchbar.TextChanged += HandleTextChanged;
            searchbar.CancelButtonClicked += HandleCancelButtonClicked;
            searchbar.Placeholder = "Search in All Columns";
            viewmodel.filtertextchanged = OnFilterChanged;

            this.View.AddSubview(searchbar);
            this.View.AddSubview(SfGrid);
        }

        void GridAutoGenerateColumns(object sender, AutoGeneratingColumnArgs e)
        {
            if (e.Column.MappingName == "BookID")
            {
                e.Column.HeaderText = "Book ID";
                e.Column.TextAlignment = UITextAlignment.Center;
            }
        }

        internal void OnFilterChanged()
        {
            if (this.SfGrid.View != null)
            {
                //Here we have used a predicate called Filter in order to display the filtered records.
                SfGrid.View.Filter = viewmodel.FilerRecords;
                SfGrid.View.RefreshFilter();
            }
        }

        void HandleCancelButtonClicked(object sender, EventArgs e)
        {
            searchbar.ResignFirstResponder();
            searchbar.SetShowsCancelButton(false, true);
            searchbar.ResignFirstResponder();
        }

        void HandleTextChanged(object sender, UISearchBarTextChangedEventArgs e)
        {
            viewmodel.FilterText = e.SearchText;
        }

        void HandleOnEditingStarted(object sender, EventArgs e)
        {
            searchbar.SetShowsCancelButton(true, true);
        }

        public override void ViewDidLayoutSubviews()
        {

            searchbar.Frame = (new CGRect(0, 0, this.View.Frame.Width, 40));
            if (UIDevice.CurrentDevice.CheckSystemVersion(7, 1))
            {
                searchbar.SizeToFit();
                this.SfGrid.Frame = new CGRect(0, searchbar.Bounds.Height, this.View.Frame.Width, this.View.Frame.Height - 40);

            }
            else
            {
                searchbar.SizeToFit();
                this.SfGrid.Frame = new CGRect(0, searchbar.Bounds.Height, this.View.Frame.Width, this.View.Frame.Height - 40);
            }

            base.ViewDidLayoutSubviews();
        }


        protected override void Dispose(bool disposing)
        {
            base.Dispose(disposing);
            SfGrid.Dispose();
        }
    }

    public partial class OptionsView : UIView
    {
        private FilterViewModel filtermodel;
        private UITableView table;
        private UISearchBar bar;
        private UITableView filterconditiontable;
        List<string> items;
        Filtering ParentView;

        static bool UserInterfaceIdiomIsPhone
        {
            get { return UIDevice.CurrentDevice.UserInterfaceIdiom == UIUserInterfaceIdiom.Phone; }
        }

        public OptionsView()
        {
            items = new List<string>();
            table = new UITableView();
            filterconditiontable = new UITableView();
            table.SectionIndexTrackingBackgroundColor = UIColor.FromRGB(0, 121, 255);
            filterconditiontable.AllowsSelection = true;
            this.AddSubview(filterconditiontable);
            this.AddSubview(table);
        }


        public OptionsView(FilterViewModel model, UISearchBar bar, Filtering parentView) : this()
        {
            this.ParentView = parentView;
            this.filtermodel = model;
            var columnnames = filtermodel.BookInfo.GetType().GetGenericArguments()[0].GetProperties();
            this.bar = bar;
            foreach (var propety in columnnames)
            {
                items.Add(propety.Name);
            }
            table.Source = new OptionsTableSource(items);
            filterconditiontable.Source = new FilterOptionsTableSource(new List<string>() {
                "Contains",
                "Equals",
                "Not Equals"
            });
            this.AddSubview(filterconditiontable);
            this.AddSubview(table);

        }

        public override void RemoveFromSuperview()
        {
            base.RemoveFromSuperview();
            filtermodel.SelectedColumn = (table.Source as OptionsTableSource).selectedItem;
            filtermodel.SelectedCondition = (filterconditiontable.Source as FilterOptionsTableSource).selecteditem;
            if (filtermodel.SelectedColumn != null && filtermodel.SelectedCondition != null)
            {
                this.bar.Placeholder = "Search " + filtermodel.SelectedColumn + " with " + filtermodel.SelectedCondition;
                if (this.bar.Text != "")
                    this.ParentView.OnFilterChanged();
            }
            else if ((filtermodel.SelectedColumn != null && filtermodel.SelectedCondition == null) ||
              (filtermodel.SelectedColumn == null && filtermodel.SelectedCondition != null))
            {
                var alert = new UIAlertView("Error", "Should Select Both ColumnName and Condition Type", null, "Cancel", null);
                alert.Frame = new CGRect(50, this.Frame.GetMidX(), 60, this.Frame.GetMidY());
                alert.Show();
            }
        }

        public override void LayoutSubviews()
        {
            table.Frame = (new CGRect(0, 0, this.Frame.Width, (this.Frame.Height / 2) + 13));
            filterconditiontable.Frame = (new CGRect(0, table.Bounds.Bottom + 2, this.Frame.Width, this.Frame.Height));
            base.LayoutSubviews();
        }

    }
}